"""Subclass of BuildTools, which is generated by wxFormBuilder."""

import wx
import wxx
import app


# Implementing BuildTools
class BuildTools(app.BuildTools):
    """
    This dialog is used for setup of build tools.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent):
        """Dialog initialization"""
        super(BuildTools, self).__init__(parent)
        import pane
        self._toolsPane = pane.BuildBinaries(self)
        if wx.__version__ >= '3.0.0.0':
            self.m_auinotebook4.AddPage(self._toolsPane, u"Binaries", False, wx.wx.NOT_FOUND)
        else:
            self.m_auinotebook4.AddPage(self._toolsPane, u"Binaries", False, wx.NullBitmap)


