# -*- coding: utf-8 -*-

from tools import edit
from tools import append_menuitem_copy
from tools import set_menu_handlers
from tools import unset_menu_handlers
from tools import clone_mnu

from ctrl import Editor
