# -*- coding: utf-8 -*-
from GitRepo import GitRepoDialog
from OpenGitRepo import OpenGitRepoDialog
from CommitGit import CommitGitDialog
from GitRemote import GitRemoteDialog
from Progress import ProgressDialog