"""Subclass of Working, which is generated by wxFormBuilder."""

import app
import sys
import os


# Implementing Working
class WorkingDialog(app.Working):
    """Dialog for showing wait"""
    def __init__(self, parent):
        """Initialize dialog"""
        super(WorkingDialog, self).__init__(parent)
        path = os.path.split(sys.argv[0])[0]
        self.m_animCtrl.LoadFile(os.path.join(path, "app/res/wait.gif"))
        self.m_animCtrl.Play()

