"""Subclass of StackFrame, which is generated by wxFormBuilder."""

from activity.files.ui import ui as ui


# Implementing StackFrame
class StackFrame(ui.StackFrame):
    """Aux pane for stack"""
    def __init__(self, parent, handler):
        """Init"""
        self._handler = handler
        super(StackFrame, self).__init__(parent)

    # Handlers for StackFrame events.
    def OnSelectStackFrame(self, event):
        """"""
        # TODO: Implement OnSelectStackFrame
        pass

    def UpdateData(self, current_thread):
        """Update info with the stack"""
        self.m_stackFrame.Clear()
        # update stack frame
        for element in current_thread.stack:
            i = self.m_stackFrame.Append(element[0])
            self.m_stackFrame.SetClientData(i, element)

    def OnActivateStackFrame(self, event):
        """"""
        # TODO: Implement OnActivateStackFrame
        pass
