"""Subclass of PdfViewPane, which is generated by wxFormBuilder."""
import wx
import app
from app.ui.pane import BasePane

# Implementing PdfViewPane
class PdfViewPane(BasePane, app.PdfViewPane):
    """Pane for visualize pdf documents"""
    def __init__(self, parent, mainframe, fileobj):
        """Init"""
        super(PdfViewPane, self).__init__(parent)
        self.m_pdfView.UsePrintDirect = False
        self.m_pdfView.SetFont( wx.Font( wx.NORMAL_FONT.GetPointSize(), 70, 94, 90, False, "Arial" ) )
        self.m_pdfView.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DDKSHADOW ) )
        self.m_pdfView.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BACKGROUND ) )
        #self.m_pdfView.SetSizerProps(expand=True, proportion=1)
        self.m_pdfView.LoadFile(fileobj._file)
        self.m_pdfView.Show()
        self.m_pdfView.Update()

