"""Subclass of TasksPane, which is generated by wxFormBuilder."""

import wx
import app


# Implementing TasksPane
class TasksPane(app.TasksPane):
    """Implements task pane"""
    def __init__(self, parent):
        """"""
        super(TasksPane, self).__init__(parent)
