from ClassDiagram import ClassDiagramDialog
from ContextItems import ContextItems
from GenerateSources import GenerateSourcesDialog
from Note import NoteDialog
from Project import ProjectDialog
from cc import *
from py import *
