"""Subclass of Expressions, which is generated by wxFormBuilder."""

from activity.files.ui import ui as ui


# Implementing Expressions
class Expressions(ui.Expressions):
    """Pane showing user expressions to be evaluated"""

    def __init__(self, parent, handler):
        """Initialize pane"""
        self._handler = handler
        super(Expressions, self).__init__(parent)

