"""Subclass of TargetsView, which is generated by wxFormBuilder."""

import wx
import app
import deco
import app.resources as rc
from app.ui.view import BaseView
from activity.targets.ui import ui
from app.ui.view import BaseView


# Implementing TargetsView
class TargetsView(BaseView, ui.TargetsView):
    """Implements target view"""
    perspective = ''

    def __init__(self, parent):
        """Initialize target"""
        super(TargetsView, self).__init__(parent)

    @deco.classproperty
    def name(cls):
        """returns the name of this view"""
        return 'Targets'

    @classmethod
    def bitmap_index(cls):
        """return the bitmap index"""
        return rc.GetBitmapIndex('target')

    @classmethod
    def bitmap(cls):
        """return the bitmap"""
        return rc.GetBitmap('target')

    # Handlers for TargetsView events.
    def OnTreeOpenFile(self, event):
        """Process open file event"""
        pass

    def OnTreeSelChanged(self, event):
        """Process on tree change selection"""
        # TODO: Implement OnTreeSelChanged
        pass

