"""Subclass of Wait, which is generated by wxFormBuilder."""

import app


# Implementing Wait
class WaitDialog(app.Wait):
    """This class shows a dialog"""
    def __init__(self, parent):
        """Initialize dialog"""
        super(WaitDialog, self).__init__(parent)
