# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

"""Subclass of PyModulePane, which is generated by wxFormBuilder."""

#import wx
import wx, wx.stc, wxx, app
from ctx import theContext as context
import tran
import pane


# Implementing PyModulePane
class NavigatorPane(app.NavigatorPane):
    """Implements navigator pane"""
    def __init__(self, parent, mainframe):
        """Intialization of method editor"""
        super(NavigatorPane, self).__init__(parent)
        self._mainframe = mainframe
        self._url = None

    def PreDelete(self):
        """Remove toolbar first in order to avoid gtk-collision at close"""
        #self.m_auiToolBar5.Destroy()

    def NavigateTo(self, str_url):
        """Do navigation"""
        self._url = str_url
        self.m_url.Clear()
        self.m_url.WriteText(str_url)
        self.m_page.LoadURL(str_url)


