# -*- coding: utf-8 -*-

from ast_explorer import ast_explorer
from log_explorer import log_explorer
from web_browser import web_browser
from check_updates import check_updates

ALL = [ast_explorer, log_explorer, web_browser, check_updates]