from EditDecorators import EditDecoratorsDialog
from PyDecorator import PyDecoratorDialog
from PyArgument import PyArgumentDialog
from PyClass import PyClassDialog
from PyFunction import PyFunctionDialog
from PyImport import PyImportDialog
from PyInheritance import PyInheritanceDialog
from PyMember import PyMemberDialog
from PyMemberMethod import PyMemberMethodDialog
from PyModule import PyModuleDialog
from PyPackage import PyPackageDialog
from PyVariable import PyVariableDialog
