from Argument import ArgumentDialog
from Class import ClassDialog
from Constructor import ConstructorDialog
from Destructor import DestructorDialog
from Enum import EnumDialog
from EnumItem import EnumItemDialog
from Function import FunctionDialog
from Inheritance import InheritanceDialog
from IsClassMethods import IsClassMethods
from Member import MemberDialog
from MemberMethod import MemberMethodDialog
from Module import ModuleDialog
from Namespace import NamespaceDialog
from Relation import RelationDialog
from SelectClasses import SelectClassesDialog
from SelectContexts import SelectContexts
from SelectFriends import SelectFriendsDialog
from SelectLibraries import SelectLibrariesDialog
from Type import TypeDialog
from UserSections import UserSectionsDialog
from Variable import VariableDialog
from VirtualMethods import VirtualMethodsDialog
from BoostPythonModule import BoostPythonModuleDialog
from Navigator import NavigatorDialog
